package de.fhdw.gaming.ipspiel23.memory;

import java.util.Optional;

import de.fhdw.gaming.core.domain.Player;
import de.fhdw.gaming.core.domain.State;

/**
 * Represents the base class for all game states offering support for memory management 
 * <p>
 * Provides access to the {@link IGameMemoryProvider} that is used to store and retrieve decisions and round data.
 * </p>
 * @param <P> The player type.
 * @param <S> The state type.
 * @param <ROUND_DATA> The round data type.
 */
public abstract class MemoryState<P extends Player<P>, S extends State<P, S>, ROUND_DATA> 
    implements IMemoryState<P, S, ROUND_DATA> {

    /**
     * needs to be preserved across states/rounds/games
     * Don*t know how to keep them otherwise, so what was the exact problem with the static solution?
     */
    private final IGameMemoryProvider memoryProvider = GameMemoryProvider.getInstance();

    @Override
    public IGameMemoryProvider getMemoryProvider() {
        return this.memoryProvider;
    }

    /**
     * Stores the given round data in the memory for the given identifier, 
     * if there is a memory instance associated with the given identifier.
     * There may be strategies that do not need a memory
     * so we don't want to create a memory for those strategies
     * only request a memory if there is one
     * @param identifier The identifier uniquely describing the memory instance 
     * for a given player and strategy combination.
     * @param roundData The round data to store.
     */
    protected void storeRoundData(final IGameMemoryIdentifier identifier, final ROUND_DATA roundData) {
        final Optional<IGameMemory<ROUND_DATA>> memory = memoryProvider.tryRequestMemoryForStrategy(identifier);
        if (memory.isPresent()) {
            memory.get().add(roundData);
        }
    }
}
